@@ -165,17 +165,18 @@ def interp_3d_points(element, d_global, num_points, scaling):
     return interpolation_points
 
 
-def add_data__extruded_frames_deformed_mesh(analysis,
-                                            case_name,
-                                            data_dict,
-                                            list_of_frames,
-                                            step,
-                                            scaling):
+def add_data__extruded_line_elms_deformed_mesh(
+        analysis,
+        case_name,
+        data_dict,
+        list_of_line_elems,
+        step,
+        scaling):
     """
     Adds a trace containing frame element extrusion mesh
     in its deformed state
     """
-    if not list_of_frames:
+    if not list_of_line_elems:
         return
     x_list = []
     y_list = []
@@ -185,9 +186,12 @@ def add_data__extruded_frames_deformed_mesh(analysis,
     k_list = []
     intensity = []
     index = 0
-    for elm in list_of_frames:
+    for elm in list_of_line_elems:
         if elm.visibility.hidden_when_extruded:
             continue
+        if elm.section.outside_shape is None:
+            continue
+
         num_points = 8
         # translations and rotations at the offset ends
         u_i = (analysis.results[case_name].node_displacements
@@ -269,10 +273,14 @@ def add_data__extruded_frames_deformed_mesh(analysis,
                 i_list.append(index + 0)
                 j_list.append(index + 2)
                 k_list.append(index + 3)
-                intensity.append(np.linalg.norm(d_global[i, :]))
-                intensity.append(np.linalg.norm(d_global[i+1, :]))
-                intensity.append(np.linalg.norm(d_global[i+1, :]))
-                intensity.append(np.linalg.norm(d_global[i, :]))
+                intensity.append(
+                    np.sqrt(d_global[i, :]@d_global[i, :]))
+                intensity.append(
+                    np.sqrt(d_global[i+1, :]@d_global[i+1, :]))
+                intensity.append(
+                    np.sqrt(d_global[i+1, :]@d_global[i+1, :]))
+                intensity.append(
+                    np.sqrt(d_global[i, :]@d_global[i, :]))
                 index += 4
     data_dict.append({
         "type": "mesh3d",
@@ -284,150 +292,33 @@ def add_data__extruded_frames_deformed_mesh(analysis,
         "j": j_list,
         "k": k_list,
         "intensity": intensity,
-        "colorbar_title": 'Displacement',
+        "colorbar": {
+            "title": "Displacement",
+            "ypad": 300
+        },
         "hoverinfo": "skip",
         "opacity": 0.65
     })
 
 
-# def add_data__extruded_steel_W_PZ_deformed_mesh(
-#         analysis, dt, list_of_endsegments, step, scaling):
-
-#     # determine the mesh3d dictionary
-#     # to append components to it
-#     # instead of creating a new mesh3d
-#     # (to have a consistent colorscale)
-#     idx = 0
-#     for k, item in enumerate(dt):
-#         if item['type'] == 'mesh3d':
-#             idx = k
-#             break
-
-#     imax = max(dt[idx]['i'])
-#     jmax = max(dt[idx]['j'])
-#     kmax = max(dt[idx]['k'])
-#     ijkmax = max((imax, jmax, kmax))
-
-#     index = ijkmax + 1
-
-#     if not list_of_endsegments:
-#         return
-#     x_list = []
-#     y_list = []
-#     z_list = []
-#     i_list = []
-#     j_list = []
-#     k_list = []
-#     intensity = []
-
-#     for elm in list_of_endsegments:
-
-#         num_points = 2
-#         # translations and rotations at the offset ends
-#         u_i = analysis.node_displacements[str(elm.n_external.uid)][step][0:3]
-#         r_i = analysis.node_displacements[str(elm.n_external.uid)][step][3:6]
-#         u_j = analysis.node_displacements[str(elm.n_internal.uid)][step][0:3]
-#         r_j = analysis.node_displacements[str(elm.n_internal.uid)][step][3:6]
-
-#         d_global, r_local = interp3D_deformation(
-#             elm, u_i, r_i, u_j, r_j, num_points)
-
-#         interpolation_points = interp3D_points(
-#             elm, d_global, r_local, num_points, scaling)
-#         x_vec = elm.parent.x_axis
-#         y_vec = elm.parent.y_axis
-#         z_vec = np.cross(x_vec, y_vec)
-#         for i in range(num_points-1):
-#             loc_i_global = interpolation_points[i, :]
-#             loc_j_global = interpolation_points[i+1, :]
-#             rot_i_local = r_local[i, :]
-#             rot_j_local = r_local[i+1, :]
-
-#             loop = elm.parent.section.mesh.halfedges
-#             for halfedge in loop:
-
-#                 z_a = halfedge.vertex.coords[0]
-#                 y_a = halfedge.vertex.coords[1]
-#                 z_b = halfedge.nxt.vertex.coords[0]
-#                 y_b = halfedge.nxt.vertex.coords[1]
-#                 defo_ia_global = z_a * z_vec + y_a * y_vec + \
-#                     (- rot_i_local[2] * y_a * x_vec +
-#                      rot_i_local[1] * z_a * x_vec
-#                      + rot_i_local[0] * y_a * z_vec
-#                      - rot_i_local[0] * z_a * y_vec
-#                      )*scaling
-#                 defo_ja_global = z_a * z_vec + y_a * y_vec + \
-#                     (- rot_j_local[2] * y_a * x_vec +
-#                      rot_j_local[1] * z_a * x_vec
-#                      + rot_j_local[0] * y_a * z_vec
-#                      - rot_j_local[0] * z_a * y_vec
-#                      )*scaling
-#                 defo_ib_global = z_b * z_vec + y_b * y_vec + \
-#                     (- rot_i_local[2] * y_b * x_vec +
-#                      rot_i_local[1] * z_b * x_vec
-#                      + rot_i_local[0] * y_b * z_vec
-#                      - rot_i_local[0] * z_b * y_vec
-#                      )*scaling
-#                 defo_jb_global = z_b * z_vec + y_b * y_vec + \
-#                     (- rot_j_local[2] * y_b * x_vec +
-#                      rot_i_local[1] * z_b * x_vec
-#                      + rot_j_local[0] * y_b * z_vec
-#                      - rot_j_local[0] * z_b * y_vec
-#                      )*scaling
-#                 loc0 = loc_i_global + defo_ia_global
-#                 loc1 = loc_j_global + defo_ja_global
-#                 loc2 = loc_j_global + defo_jb_global
-#                 loc3 = loc_i_global + defo_ib_global
-#                 x_list.append(loc0[0])
-#                 y_list.append(loc0[1])
-#                 z_list.append(loc0[2])
-#                 x_list.append(loc1[0])
-#                 y_list.append(loc1[1])
-#                 z_list.append(loc1[2])
-#                 x_list.append(loc2[0])
-#                 y_list.append(loc2[1])
-#                 z_list.append(loc2[2])
-#                 x_list.append(loc3[0])
-#                 y_list.append(loc3[1])
-#                 z_list.append(loc3[2])
-#                 i_list.append(index + 0)
-#                 j_list.append(index + 1)
-#                 k_list.append(index + 2)
-#                 i_list.append(index + 0)
-#                 j_list.append(index + 2)
-#                 k_list.append(index + 3)
-#                 intensity.append(np.linalg.norm(d_global[0, :]))
-#                 intensity.append(np.linalg.norm(d_global[1, :]))
-#                 intensity.append(np.linalg.norm(d_global[1, :]))
-#                 intensity.append(np.linalg.norm(d_global[0, :]))
-#                 index += 4
-
-#     dt[idx]['x'].extend(x_list)
-#     dt[idx]['y'].extend(y_list)
-#     dt[idx]['z'].extend(z_list)
-#     dt[idx]['i'].extend(i_list)
-#     dt[idx]['j'].extend(j_list)
-#     dt[idx]['k'].extend(k_list)
-#     dt[idx]['intensity'].extend(intensity)
-
-
-def add_data__frames_deformed(analysis,
-                              case_name,
-                              data_dict,
-                              list_of_frames,
-                              step,
-                              scaling):
+def add_data__line_elms_deformed(
+        analysis,
+        case_name,
+        data_dict,
+        list_of_line_elems,
+        step,
+        scaling):
     """
     Adds a trace containing frame element centroidal axis lines
     in their deformed state
     """
-    if not list_of_frames:
+    if not list_of_line_elems:
         return
     x_list: list[Optional[float]] = []
     y_list: list[Optional[float]] = []
     z_list: list[Optional[float]] = []
 
-    for elm in list_of_frames:
+    for elm in list_of_line_elems:
         if elm.visibility.hidden_at_line_plots:
             continue
         num_points = 8
@@ -468,27 +359,28 @@ def add_data__frames_deformed(analysis,
         "hoverinfo": "skip",
         "line": {
             "width": 5,
-            "color": graphics_common.BEAM_MESH_COLOR
+            "color": graphics_common.FRAME_COLOR
         }
     })
 
 
-def add_data__frames_offsets_deformed(analysis,
-                                      case_name,
-                                      data_dict,
-                                      list_of_frames,
-                                      step,
-                                      scaling):
+def add_data__line_elm_offsets_deformed(
+        analysis,
+        case_name,
+        data_dict,
+        list_of_line_elems,
+        step,
+        scaling):
     """
     Adds a trace containing frame element rigid offset lines
     in their deformed state
     """
-    if not list_of_frames:
+    if not list_of_line_elems:
         return
     x_list: list[Optional[float]] = []
     y_list: list[Optional[float]] = []
     z_list: list[Optional[float]] = []
-    for elm in list_of_frames:
+    for elm in list_of_line_elems:
         if np.array_equal(elm.geomtransf.offset_i, np.zeros(3)):
             if np.array_equal(elm.geomtransf.offset_j, np.zeros(3)):
                 continue
@@ -542,7 +434,7 @@ def add_data__frames_offsets_deformed(analysis,
     })
 
 
-def add_data__frames_undeformed(data_dict, list_of_frames):
+def add_data__frames_undeformed(data_dict, list_of_line_elems):
     """
     Adds a trace containing frame element centroidal axis lines
     """
@@ -550,7 +442,7 @@ def add_data__frames_undeformed(data_dict, list_of_frames):
     y_list: list[Optional[float]] = []
     z_list: list[Optional[float]] = []
 
-    for elm in list_of_frames:
+    for elm in list_of_line_elems:
 
         p_i = np.array(elm.nodes[0].coords) + elm.geomtransf.offset_i
         p_j = np.array(elm.nodes[1].coords) + elm.geomtransf.offset_j
@@ -682,7 +574,10 @@ def show_deformed_shape(analysis,
                         scaling,
                         extrude,
                         camera=None,
-                        subset_model=None):
+                        subset_model=None,
+                        animation=True,
+                        init_step=0,
+                        step_skip=0):
 
     """
     Visualize the model in its deformed state
@@ -695,65 +590,157 @@ def show_deformed_shape(analysis,
       extrude (bool): wether to extrude frame elements
       camera (dict): custom positioning of the camera
     """
+
     if subset_model:
+        # if a subset model is specified, only show its components
         mdl = subset_model
     else:
+        # otherwise show the entire model
         mdl = analysis.mdl
+
     # calculate a nice scaling factor if 0.00 is passed
-    if scaling == 0:
-        scaling = get_auto_scaling_deformation(analysis, case_name, mdl, step)
+    if scaling == 0.00:
+        scaling = get_auto_scaling_deformation(
+            analysis, case_name, mdl, step)
 
-    layout = graphics_common_3d.global_layout(camera)
+    # instantiate layout and datastructures
+    layout = graphics_common_3d.global_layout(mdl, camera)
     data_dict: list[dict[str, object]] = []
+    frame_data_dict: list[list[dict[str, object]]] = []
 
-    list_of_frames = [elm for elm in mdl.list_of_beamcolumn_elements()
-                      if not elm.visibility.skip_opensees_definition]
-
-    # list_of_steel_W_panel_zones = \
-    #     mdl.list_of_steel_W_panel_zones()
+    # gather lists of associated objects
+    list_of_line_elems = mdl.list_of_beamcolumn_elements()
     list_of_primary_nodes = mdl.list_of_primary_nodes()
     list_of_internal_nodes = mdl.list_of_internal_nodes()
     # list_of_parent_nodes = mdl.list_of_parent_nodes()
-    # list_of_release_nodes = \
-    #     [x.node_i for x in mdl.list_of_endreleases()]
 
-    # if list_of_parent_nodes:
-    #     add_data__nodes_deformed(
-    #         analysis, dt, list_of_parent_nodes, step, scaling)
+    # add data for the global axes
+    ref_len = mdl.reference_length()
 
-    if not extrude:
-        # draw the nodes
-        add_data__nodes_deformed(
-            analysis, case_name, data_dict, list_of_primary_nodes,
-            step, scaling, 'free')
-        add_data__nodes_deformed(
-            analysis, case_name, data_dict, list_of_internal_nodes,
-            step, scaling, 'internal')
-    #     add_data__nodes_deformed(
-    #         analysis, dt, list_of_release_nodes, step, scaling,
-    #         color=graphics_common.NODE_INTERNAL_COLOR,
-    #         marker=graphics_common_3d.node_marker['pinned'][0],
-    #         size=graphics_common_3d.node_marker['pinned'][1])
-        # draw the frames as lines
-        add_data__frames_offsets_deformed(
-            analysis, case_name, data_dict, list_of_frames, step, scaling)
-        add_data__frames_deformed(
-            analysis, case_name, data_dict, list_of_frames, step, scaling)
-        # we also add axes so that we can see 2D plots
-        ref_len = mdl.reference_length()
-        add_data__global_axes(data_dict, ref_len)
+    add_data__global_axes(data_dict, ref_len)
+    if animation:
+        first_step = init_step
     else:
-        # draw the extruded frames
-        add_data__extruded_frames_deformed_mesh(
-            analysis, case_name, data_dict, list_of_frames, step, scaling)
-        # add_data__extruded_steel_W_PZ_deformed_mesh(
-        #     analysis, dt, list_of_steel_W_panel_zones, step, scaling)
+        first_step = step
 
-    fig_datastructure = dict(data=data_dict, layout=layout)
-    fig = go.Figure(fig_datastructure)
+    add_data__nodes_deformed(
+        analysis, case_name, data_dict, list_of_primary_nodes,
+        first_step, scaling, 'free')
+    add_data__nodes_deformed(
+        analysis, case_name, data_dict, list_of_internal_nodes,
+        first_step, scaling, 'internal')
+    # add data for the frame elements as lines
+    add_data__line_elm_offsets_deformed(
+        analysis, case_name, data_dict,
+        list_of_line_elems, first_step, scaling)
+    add_data__line_elms_deformed(
+        analysis, case_name, data_dict,
+        list_of_line_elems, first_step, scaling)
+    # add data for the extruded frame elements
+    if extrude:
+        add_data__extruded_line_elms_deformed_mesh(
+            analysis, case_name, data_dict,
+            list_of_line_elems, first_step, scaling)
+
+    if animation:
+        step_of_frame = []
+        for j in range(first_step, step, step_skip+1):
+            step_of_frame.append(j)
+        for _, j in enumerate(step_of_frame):
+            frame_data_dict.append([])
+            add_data__global_axes(frame_data_dict[-1], ref_len)
+            # add_data__global_axes(frame_data_dict[0], ref_len)
+            add_data__nodes_deformed(
+                analysis, case_name, frame_data_dict[-1],
+                list_of_primary_nodes,
+                j, scaling, 'free')
+            add_data__nodes_deformed(
+                analysis, case_name, frame_data_dict[-1],
+                list_of_internal_nodes,
+                j, scaling, 'internal')
+            # add data for the frame elements as lines
+            add_data__line_elm_offsets_deformed(
+                analysis, case_name, frame_data_dict[-1],
+                list_of_line_elems, j, scaling)
+            add_data__line_elms_deformed(
+                analysis, case_name, frame_data_dict[-1],
+                list_of_line_elems, j, scaling)
+            # add data for the extruded frame elements
+            if extrude:
+                add_data__extruded_line_elms_deformed_mesh(
+                    analysis, case_name, frame_data_dict[-1],
+                    list_of_line_elems, j, scaling)
+
+    # create the plot
+
+    def frame_args(duration):
+        return {
+                "frame": {"duration": duration},
+                "mode": "immediate",
+                "fromcurrent": True,
+                "transition": {"duration": duration, "easing": "linear"},
+            }
+
+    if animation:
+        fig_datastructure = dict(
+            data=data_dict,
+            layout=layout,
+            frames=[go.Frame(data=frame_data_dict[j],
+                             name=str(step_of_frame[j]))
+                    for j in range(len(step_of_frame))])
+        fig = go.Figure(fig_datastructure)
+        sliders = [
+                    {
+                        "pad": {"b": 10, "t": 60},
+                        "len": 0.9,
+                        "x": 0.1,
+                        "y": 0,
+                        "steps": [
+                            {
+                                "args": [[f.name], frame_args(0)],
+                                "label": str(k),
+                                "method": "animate",
+                            }
+                            for k, f in enumerate(fig.frames)
+                        ],
+                    }
+                ]
+        # Layout
+        fig.update_layout(
+            updatemenus=[
+                {
+                    "buttons": [
+                        {
+                            "args": [None, frame_args(50)],
+                            "label": "&#9654;",  # play symbol
+                            "method": "animate",
+                        },
+                        {
+                            "args": [[None], frame_args(0)],
+                            "label": "&#9724;",  # pause symbol
+                            "method": "animate",
+                        },
+                    ],
+                    "direction": "left",
+                    "pad": {"r": 10, "t": 70},
+                    "type": "buttons",
+                    "x": 0.1,
+                    "y": 0,
+                }
+            ],
+            sliders=sliders
+        )
+    else:
+        fig_datastructure = dict(
+            data=data_dict,
+            layout=layout)
+        fig = go.Figure(fig_datastructure)
+
+    # show the plot (if it's not a test)
     if "pytest" not in sys.modules:
         fig.show()
 
+    # return plot-related metadata
     metadata = {'scaling': scaling}
     return metadata
 
@@ -799,8 +786,6 @@ def show_basic_forces(analysis,
           model.
     """
     # TODO: what is going on with the scaling factors?...
-    layout = graphics_common_3d.global_layout(camera)
-    data_dict: list[dict[str, object]] = []
 
     if subset_model:
         mdl = subset_model
@@ -809,6 +794,9 @@ def show_basic_forces(analysis,
     list_of_line_elements = [elm for elm in mdl.list_of_beamcolumn_elements()
                              if not elm.visibility.skip_opensees_definition]
 
+    layout = graphics_common_3d.global_layout(mdl, camera)
+    data_dict: list[dict[str, object]] = []
+
     # draw the frames
     add_data__frames_undeformed(
         data_dict, list_of_line_elements)
@@ -1278,8 +1266,6 @@ def show_basic_forces_combo(
           model.
     """
     # TODO: merge code repetitions with the previous function
-    layout = graphics_common_3d.global_layout(camera)
-    data_dict: list[dict[str, object]] = []
 
     if subset_model:
         mdl = subset_model
@@ -1288,6 +1274,9 @@ def show_basic_forces_combo(
     list_of_line_elements = [elm for elm in mdl.list_of_beamcolumn_elements()
                              if not elm.visibility.skip_opensees_definition]
 
+    layout = graphics_common_3d.global_layout(mdl, camera)
+    data_dict: list[dict[str, object]] = []
+
     # draw the frames
     add_data__frames_undeformed(
         data_dict, list_of_line_elements)
