@@ -488,6 +488,67 @@ def add_data__frames(data_dict, mdl, load_case):
         })
 
 
+def add_data__twonodelinks(data_dict, mdl):
+    """
+    Adds a trace containing twonodelink elements
+    Arguments:
+      data_dict (dict): dictionary containing figure data
+      mdl (Model): the model to be visualized
+    """
+    link_elems = mdl.list_of_twonodelink_elements()
+    if not link_elems:
+        return
+    x_list: list[Optional[float]] = []
+    y_list: list[Optional[float]] = []
+    z_list: list[Optional[float]] = []
+    customdata_list = []
+    for elm in link_elems:
+        p_i: nparr = np.array(elm.nodes[0].coords)
+        p_j: nparr = np.array(elm.nodes[1].coords)
+        x_list.extend(
+            (p_i[0], p_j[0], None)
+        )
+        y_list.extend(
+            (p_i[1], p_j[1], None)
+        )
+        z_list.extend(
+            (p_i[2], p_j[2], None)
+        )
+        customdata_list.append(
+            (elm.uid,
+             elm.nodes[0].uid,
+             elm.parent_component.uid)
+        )
+        customdata_list.append(
+            (elm.uid,
+             elm.nodes[1].uid,
+             elm.parent_component.uid)
+        )
+        customdata_list.append(
+            (None,)*3
+        )
+
+    customdata: nparr = np.array(customdata_list, dtype='object')
+    data_dict.append({
+        "name": "TwoNodeLink elements",
+        "type": "scatter3d",
+        "mode": "lines",
+        "x": x_list,
+        "y": y_list,
+        "z": z_list,
+        # "text": section_names,
+        "customdata": customdata,
+        "hovertemplate": 'Section: %{text}<br>' +
+        '<extra>Element: %{customdata[0]:d}<br>' +
+        'Node @ this end: %{customdata[1]:d}<br>'
+        'Parent: %{customdata[2]}</extra>',
+        "line": {
+            "width": 5,
+            "color": graphics_common.LINK_COLOR
+        }
+    })
+
+
 def add_data__frame_offsets(data_dict, mdl):
     """
     Adds a trace containing frame element rigid offset lines
@@ -661,7 +722,9 @@ def add_data__global_axes(data_dict, ref_len):
     colors.extend(["red"]*3)
     colors.extend(["green"]*3)
     colors.extend(["blue"]*3)
-    data_dict.append({
+    # we add it twice in order for the animations to work
+    # see https://plotly.com/python/animations/
+    data_dict.extend([{
         "type": "scatter3d",
         "mode": "lines",
         "x": x_list,
@@ -672,7 +735,7 @@ def add_data__global_axes(data_dict, ref_len):
             "width": 3,
             "color": colors
         }
-    })
+    }]*2)
 
 
 def add_data__extruded_frames_mesh(data_dict, mdl):
@@ -749,8 +812,6 @@ def show(mdl: Model,
          extrude=False,
          offsets=True,
          global_axes=True,
-         diaphragm_lines=True,
-         tributary_areas=True,
          parent_nodes=True,
          frame_axes=False,
          zerolength_axes=False,
@@ -776,7 +837,7 @@ def show(mdl: Model,
       camera (dict): custom positioning of the camera
     """
 
-    layout = graphics_common_3d.global_layout(camera)
+    layout = graphics_common_3d.global_layout(mdl, camera)
     data_dict: list[dict[str, object]] = []
 
     ref_len = mdl.reference_length()
@@ -784,8 +845,7 @@ def show(mdl: Model,
     # plot the nodes
 
     add_data__nodes(data_dict, mdl, load_case)
-    if not extrude:
-        add_data__internal_nodes(data_dict, mdl, load_case)
+    add_data__internal_nodes(data_dict, mdl, load_case)
 
     # global axes
     if global_axes:
@@ -813,7 +873,7 @@ def show(mdl: Model,
     # plot the rigid offsets
     if offsets:
         add_data__frame_offsets(data_dict, mdl)
-
+    add_data__twonodelinks(data_dict, mdl)
     # plot the parent nodes
     if parent_nodes:
         if load_case:
